@@ -7,7 +7,7 @@
  * Author: Graeme W. Gill
  * Date:   2007/10/10
  *
- * Copyright 2006 - 2007 Graeme W. Gill
+ * Copyright 2006 - 2013 Graeme W. Gill
  * All rights reserved.
  *
  * (Based on usbio.c)
@@ -16,6 +16,13 @@
  * see the License2.txt file for licencing details.
  */
 
+/*
+ * TTBD:
+ *		As usual, Apple seem to have deprecated the routines used here.
+ *		See IOHIDDeviceGetReportWithCallback & IOHIDDeviceSetReportWithCallback
+ *		for info on the replacements.
+ */
+
 /* These routines supliement the class code in ntio.c and unixio.c */
 /* with HID specific access routines for devices running on operating */
 /* systems where this is desirable. */
@@ -32,26 +39,20 @@
 #ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
+#else
+#include "sa_config.h"
 #endif
 #include "numsup.h"
 #include "xspect.h"
 #include "insttypes.h"
-#include "icoms.h"
 #include "conv.h"
-#include "usbio.h"
-
-#ifdef USE_LIBUSB1
-# include "libusb.h"
-#else
-# include "usb.h"
-#endif
-#include "hidio.h"
+#include "icoms.h"
 
 #if defined(NT)
 #include <setupapi.h>
 #endif
 
-#if defined(UNIX) && !defined(__APPLE__)
+#if defined(UNIX_X11)
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -61,24 +62,14 @@
 #include <sys/types.h> 
 #include <usbhid.h> 
 #else	/* assume Linux */ 
-#include <asm/types.h>
-#include <linux/hiddev.h>
+# include <asm/types.h>
+# include <linux/hiddev.h>
 #endif
 #ifndef HID_MAX_USAGES		/* Workaround Linux Bug ? */
 # define HID_MAX_USAGES 1024
 #endif
 #endif
 
-#undef DEBUG
-
-
-#if defined(DEBUG)
-#define DBG(xxx) fprintf xxx ;
-#define dbgo stderr
-#else
-#define DBG(xxx) 
-#endif	/* DEBUG */
-
 #if defined(NT)
 
 /* Declartions to enable HID access without using the DDK */
@@ -118,10 +109,10 @@ static int setup_dyn_calls() {
 
 /* Add paths to USB connected instruments, to the existing */
 /* icompath paths in the icoms structure. */
-void hid_get_paths(
-struct _icoms *p 
-) {
-#ifdef ENABLE_USB
+/* return com error */
+int hid_get_paths(icompaths *p) {
+
+	a1logd(p->log, 8, "hid_get_paths: called\n");
 
 #if defined(NT)
 	{
@@ -133,11 +124,12 @@ struct _icoms *p
 		PSP_DEVICE_INTERFACE_DETAIL_DATA pdidd = (PSP_DEVICE_INTERFACE_DETAIL_DATA)buf;
 		SP_DEVINFO_DATA dinfod;
 		int i;
-		unsigned short VendorID = 0, ProductID = 0;
+		unsigned int VendorID = 0, ProductID = 0;
 	
 		/* Make sure we've dynamically linked */
 		if (setup_dyn_calls() == 0) {
-        	error("Dynamic linking to hid.dll failed");
+        	a1loge(p->log, ICOM_SYS, "hid_get_paths() Dynamic linking to hid.dll failed\n");
+			return ICOM_SYS;
 		}
 
 		/* Get the device interface GUID for HIDClass devices */
@@ -145,8 +137,10 @@ struct _icoms *p
 
 		/* Return device information for all devices of the HID class */
 		hdinfo = SetupDiGetClassDevs(&HidGuid, NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE); 
-		if (hdinfo == INVALID_HANDLE_VALUE)
-        	error("SetupDiGetClassDevs failed");
+		if (hdinfo == INVALID_HANDLE_VALUE) {
+        	a1loge(p->log, ICOM_SYS, "hid_get_paths() SetupDiGetClassDevs failed\n");
+			return ICOM_SYS;
+		}
 	
 		/* Get each devices interface data in turn */
 		did.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
@@ -160,11 +154,14 @@ struct _icoms *p
 				if (GetLastError() == ERROR_NO_MORE_ITEMS) {
 					break;
 				}
-				error("SetupDiEnumDeviceInterfaces failed");
+	        	a1loge(p->log, ICOM_SYS, "hid_get_paths() SetupDiEnumDeviceInterfaces failed\n");
+				return ICOM_SYS;
 			}
 			if (SetupDiGetDeviceInterfaceDetail(hdinfo, &did, pdidd, DIDD_BUFSIZE, NULL, &dinfod)
-			                                                                                == 0)
-				error("SetupDiGetDeviceInterfaceDetail failed");
+			                                                                                == 0) {
+	        	a1loge(p->log, ICOM_SYS, "hid_get_paths() SetupDiGetDeviceInterfaceDetail failed\n");
+				return ICOM_SYS;
+			}
 
 			/* Extract the vid and pid from the device path */
 			{
@@ -180,7 +177,7 @@ struct _icoms *p
 						break;
 					memcpy(buf, cp + 4, 4);
 					buf[4] = '\000';
-					if (sscanf(buf, "%hx", &VendorID) != 1)
+					if (sscanf(buf, "%x", &VendorID) != 1)
 						break;
 					if ((cp = strchr(pdidd->DevicePath, 'p')) == NULL)
 						break;
@@ -190,60 +187,49 @@ struct _icoms *p
 						break;
 					memcpy(buf, cp + 4, 4);
 					buf[4] = '\000';
-					if (sscanf(buf, "%hx", &ProductID) != 1)
+					if (sscanf(buf, "%x", &ProductID) != 1)
 						break;
 					gotid = 1;
 					break;
 				}
 				if (!gotid) {
-					if (p->debug) fprintf(stderr,"found HID device '%s', inst %d but unable get PID and VID\n",pdidd->DevicePath, dinfod.DevInst);
+					a1logd(p->log, 1, "found HID device '%s', inst %d but unable get PID and VID\n",pdidd->DevicePath, dinfod.DevInst);
 					continue;
 				}
 			}
 
 			/* If it's a device we're looking for */
-			if ((itype = inst_usb_match(VendorID, ProductID)) != instUnknown) {
+			if ((itype = inst_usb_match(VendorID, ProductID, 0)) != instUnknown) {
+				struct hid_idevice *hidd;
 				char pname[400];
 		
-				if (p->debug) fprintf(stderr,"found HID device '%s', inst %d that we're looking for\n",pdidd->DevicePath, dinfod.DevInst);
-
 				/* Create human readable path/identification */
 				sprintf(pname,"hid:/%d (%s)", dinfod.DevInst, inst_name(itype));
-		
-				/* Add the path to the list */
-				if (p->paths == NULL) {
-					if ((p->paths = (icompath **)calloc(sizeof(icompath *), 1 + 1)) == NULL)
-						error("icoms: calloc failed!");
-				} else {
-					if ((p->paths = (icompath **)realloc(p->paths,
-					                     sizeof(icompath *) * (p->npaths + 2))) == NULL)
-						error("icoms: realloc failed!");
-					p->paths[p->npaths+1] = NULL;
+
+				a1logd(p->log, 2, "found HID device '%s', inst %d that we're looking for\n",pdidd->DevicePath, dinfod.DevInst);
+
+				if ((hidd = (struct hid_idevice *) calloc(sizeof(struct hid_idevice), 1)) == NULL) {
+		        	a1loge(p->log, ICOM_SYS, "hid_get_paths() calloc failed!\n");
+					return ICOM_SYS;
 				}
-				if ((p->paths[p->npaths] = calloc(sizeof(icompath), 1)) == NULL)
-					error("icoms: calloc failed!");
-				p->paths[p->npaths]->vid = VendorID;
-				p->paths[p->npaths]->pid = ProductID;
-				p->paths[p->npaths]->dev = NULL;		/* Make sure it's NULL */
-				if ((p->paths[p->npaths]->hev = (hid_device *)calloc(sizeof(hid_device), 1))
-				                                                                     == NULL)
-					error("icoms: calloc failed!");
-				if ((p->paths[p->npaths]->hev->dpath = strdup(pdidd->DevicePath)) == NULL)
-					error("icoms: calloc failed!");
-				p->paths[p->npaths]->itype = itype;
-				if ((p->paths[p->npaths]->path = strdup(pname)) == NULL)
-					error("icoms: strdup failed!");
-				p->npaths++;
-				p->paths[p->npaths] = NULL;
+				if ((hidd->dpath = strdup(pdidd->DevicePath)) == NULL) {
+		        	a1loge(p->log, ICOM_SYS, "hid_get_paths() calloc failed!\n");
+					return ICOM_SYS;
+				}
+
+				/* Add the path to the list */
+				p->add_hid(p, pname, VendorID, ProductID, 0, hidd, itype);
+
 			} else {
-				if (p->debug) fprintf(stderr,"found HID device '%s', inst %d but not one we're looking for\n",pdidd->DevicePath, dinfod.DevInst);
+				a1logd(p->log, 6, "found HID device '%s', inst %d but not one we're looking for\n",pdidd->DevicePath, dinfod.DevInst);
 			}
 		}
 
 		/* Now we're done with the hdifo */
-		if (SetupDiDestroyDeviceInfoList(hdinfo) == 0)
-        	error("SetupDiDestroyDeviceInfoList failed");
-
+		if (SetupDiDestroyDeviceInfoList(hdinfo) == 0) {
+        	a1loge(p->log, ICOM_SYS, "SetupDiDestroyDeviceInfoList failed\n");
+			return ICOM_SYS;
+		}
 	}
 #endif /* NT */
 
@@ -255,19 +241,23 @@ struct _icoms *p
 
 		/* Get dictionary of HID devices */
     	if ((sdict = IOServiceMatching(kIOHIDDeviceKey)) == NULL) {
-        	warning("IOServiceMatching returned a NULL dictionary");
+        	a1loge(p->log, ICOM_SYS, "hid_get_paths() IOServiceMatching returned a NULL dictionary\n");
+			return ICOM_SYS;
 		}
 
 		/* Init itterator to find matching types. Consumes sdict reference */
 		if ((kstat = IOServiceGetMatchingServices(kIOMasterPortDefault, sdict, &mit))
-			                                                          != KERN_SUCCESS) 
-        	error("IOServiceGetMatchingServices returned %d", kstat);
+			                                                          != KERN_SUCCESS) { 
+        	a1loge(p->log, ICOM_SYS, "hid_get_paths() IOServiceGetMatchingServices returned %d\n", kstat);
+			return ICOM_SYS;
+		}
 
 		/* Find all the matching HID devices */
 		for (;;) {
 			io_object_t ioob;						/* HID object found */
 			CFNumberRef vref, pref;					/* HID Vendor and Product ID propeties */
-			unsigned int vid = 0, pid = 0;
+			CFNumberRef lidpref;					/* Location ID properties */
+			unsigned int vid = 0, pid = 0, lid = 0;
 			instType itype;
 
 		    if ((ioob = IOIteratorNext(mit)) == 0)
@@ -285,42 +275,32 @@ struct _icoms *p
 				CFNumberGetValue(pref, kCFNumberIntType, &pid);
 			    CFRelease(pref);
 			}
+			if ((lidpref = IORegistryEntryCreateCFProperty(ioob, CFSTR("LocationID"),
+			                                         kCFAllocatorDefault,kNilOptions)) != 0) {
+				CFNumberGetValue(lidpref, kCFNumberIntType, &lid);
+			    CFRelease(lidpref);
+			}
 
 			/* If it's a device we're looking for */
-			if ((itype = inst_usb_match(vid, pid)) != instUnknown) {
+			if ((itype = inst_usb_match(vid, pid, 0)) != instUnknown) {
+				struct hid_idevice *hidd;
 				char pname[400];
-				if (p->debug) fprintf(stderr,"found HID device '%s' that we're looking for\n",inst_name(itype));
+
+	        	a1logd(p->log, 2, "found HID device '%s' lid 0x%x that we're looking for\n",inst_name(itype), lid);
 
 				/* Create human readable path/identification */
-				/* (There seems to be no easy way of creating an interface no, without */
-				/*  heroic efforts looking through the IO registry, so don't try.) */
-				sprintf(pname,"hid: (%s)", inst_name(itype));
+				sprintf(pname,"hid%d: (%s)", lid >> 20, inst_name(itype));
 		
-				/* Add the path to the list */
-				if (p->paths == NULL) {
-					if ((p->paths = (icompath **)calloc(sizeof(icompath *), 1 + 1)) == NULL)
-						error("icoms: calloc failed!");
-				} else {
-					if ((p->paths = (icompath **)realloc(p->paths,
-					                     sizeof(icompath *) * (p->npaths + 2))) == NULL)
-						error("icoms: realloc failed!");
-					p->paths[p->npaths+1] = NULL;
+				if ((hidd = (struct hid_idevice *)calloc(sizeof(struct hid_idevice), 1)) == NULL) {
+		        	a1loge(p->log, ICOM_SYS, "hid_get_paths calloc failed!\n");
+					return ICOM_SYS;
 				}
-				if ((p->paths[p->npaths] = calloc(sizeof(icompath), 1)) == NULL)
-					error("icoms: calloc failed!");
-				p->paths[p->npaths]->vid = vid;
-				p->paths[p->npaths]->pid = pid;
-				p->paths[p->npaths]->dev = NULL;		/* Make sure it's NULL */
-				if ((p->paths[p->npaths]->hev = (hid_device *)calloc(sizeof(hid_device), 1))
-				                                                                     == NULL)
-					error("icoms: calloc failed!");
-				p->paths[p->npaths]->hev->ioob = ioob;
+				hidd->lid = lid;
+				hidd->ioob = ioob;
 				ioob = 0;			/* Don't release it */
-				p->paths[p->npaths]->itype = itype;
-				if ((p->paths[p->npaths]->path = strdup(pname)) == NULL)
-					error("icoms: strdup failed!");
-				p->npaths++;
-				p->paths[p->npaths] = NULL;
+
+				/* Add the path to the list */
+				p->add_hid(p, pname, vid, pid, 0, hidd, itype);
 			}
 			if (ioob != 0)		/* If we haven't kept it */
 			    IOObjectRelease(ioob);		/* Release found object */
@@ -329,7 +309,7 @@ struct _icoms *p
 	}
 #endif /* __APPLE__ */
 
-#if defined(UNIX) && !defined(__APPLE__)
+#if defined(UNIX_X11)
 
 	/* This is how we'd go about adding HID support for Linux, IF it */
 	/* was actually capable of communicating application composed reports - */
@@ -362,68 +342,31 @@ struct _icoms *p
 					strcpy(dpath, devds[i]);
 					strcat(dpath, "/");
 					strcat(dpath, dentry->d_name);
-//printf("~1 found hid device '%s'\n",dpath);
+//					a1logd(p->log, 8, "found hid device '%s'\n",dpath);
 					/* Open it and see what VID and PID it is */
 					if ((fd = open(dpath, O_RDONLY)) >= 0) {
 						struct hiddev_devinfo hidinfo;
 						if (ioctl(fd, HIDIOCGDEVINFO, &hidinfo) < 0)
-							warning("Unable to get HID info for '%s'",dpath);
+					       	a1logd(p->log, 1, "Unable to get HID info for '%s'\n",dpath);
 						else {
-//printf("~1 busnum = %d, devnum = %d, vid = 0x%x, pid = 0x%x\n",
-//hidinfo.busnum, hidinfo.devnum, hidinfo.vendor, hidinfo.product);
+//							a1logd(p->log, 8,"busnum = %d, devnum = %d, vid = 0x%x, pid = 0x%x\n",
+//							hidinfo.busnum, hidinfo.devnum, hidinfo.vendor, hidinfo.product);
 						}
 						close(fd);
 					}
 // More hotplug/udev magic needed to make the device accesible !
-//else printf("~1 failed to open '%s' err %d\n",dpath,fd);
+//else a1logd(p->log, 8,"failed to open '%s' err %d\n",dpath,fd);
 				}
 			}
 			closedir(dir);
 		}
 	}
 #endif /* NEVER */
-#endif /* UNIX & ! APPLE */
+#endif /* UNIX_X11 */
 
-#endif /* ENABLE_USB */
-}
+	a1logd(p->log, 8, "icoms_get_paths: returning %d paths and ICOM_OK\n",p->npaths);
 
-/* Cleanup and then free an hev entry */
-void hid_del_hid_device(hid_device *hev) {
-
-	if (hev == NULL)
-		return;
-
-#if defined(NT)
-	if (hev->dpath != NULL)
-		free(hev->dpath);
-#endif /* NT */
-#ifdef __APPLE__
-	if (hev->ioob != 0)
-	    IOObjectRelease(hev->ioob);		/* Release found object */
-#endif /* __APPLE__ */
-
-	free(hev);
-}
-
-/* Return the instrument type if the port number is HID, */
-/* and instUnknown if it is not. */
-instType hid_is_hid_portno(
-	icoms *p, 
-	int port		/* Enumerated port number, 1..n */
-) {
-	
-	if (p->paths == NULL)
-		p->get_paths(p);
-
-	if (port <= 0 || port > p->npaths)
-		error("icoms - set_ser_port: port number out of range!");
-
-#ifdef ENABLE_USB
-	if (p->paths[port-1]->hev != NULL)
-		return p->paths[port-1]->itype;
-#endif /* ENABLE_USB */
-
-	return instUnknown;
+	return ICOM_OK;
 }
 
 /*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -432,8 +375,7 @@ instType hid_is_hid_portno(
 /* If we don't do this, the port and/or the device may be left in an unusable state. */
 void hid_close_port(icoms *p) {
 
-#ifdef ENABLE_USB
-	if (p->debug) fprintf(stderr,"hid_close_port() called\n");
+	a1logd(p->log, 8, "hid_close_port: called\n");
 
 	if (p->is_open && p->hidd != NULL) {
 
@@ -446,84 +388,45 @@ void hid_close_port(icoms *p) {
 	    IOObjectRelease(p->hidd->port);
 		p->hidd->port = 0;
 
-		CFRelease(p->hidd->evsrc);
+		if (p->hidd->evsrc != NULL)
+			CFRelease(p->hidd->evsrc);
 		p->hidd->evsrc = NULL;
 
 		p->hidd->rlr = NULL;
 
-		if ((*p->hidd->device)->close(p->hidd->device) != kIOReturnSuccess)
-			error("Closing HID port '%s' failed",p->ppath->path);
+		if ((*p->hidd->device)->close(p->hidd->device) != kIOReturnSuccess) {
+			a1loge(p->log, ICOM_SYS, "hid_close_port: closing HID port '%s' failed",p->name);
+			return;
+		}
 
-		if ((*p->hidd->device)->Release(p->hidd->device) != kIOReturnSuccess)
-			warning("Releasing HID port '%s' failed",p->ppath->path);
+		if ((*p->hidd->device)->Release(p->hidd->device) != kIOReturnSuccess) {
+			a1loge(p->log, ICOM_SYS, "hid_close_port: Releasing HID port '%s' failed",p->name);
+		}
 		p->hidd->device = NULL;
 #endif /* __APPLE__ */
 
 		p->is_open = 0;
-		if (p->debug) fprintf(stderr,"hid port has been released and closed\n");
-	}
-
-	if (p->ppath != NULL) {
-		if (p->ppath->path != NULL)
-			free(p->ppath->path);
-		free(p->ppath);
-		p->ppath = NULL;
+		a1logd(p->log, 8, "hid_close_port: has been released and closed\n");
 	}
 
 	/* Find it and delete it from our static cleanup list */
 	usb_delete_from_cleanup_list(p);
-
-#endif /* ENABLE_USB */
 }
 
 
 /* Open an HID port for all our uses. */
-static void hid_open_port(
+/* This always re-opens the port */
+/* return icom error */
+static int hid_open_port(
 icoms *p,
-int    port,			/* USB com port, 1 - N, 0 for no change. */
 icomuflags hidflags,	/* Any special handling flags */
 int retries,			/* > 0 if we should retry set_configuration (100msec) */
 char **pnames			/* List of process names to try and kill before opening */
 ) {
-	if (p->debug) fprintf(stderr,"icoms: About to open the USB port\n");
-
-	if (port >= 1) {
-		if (p->is_open && port != p->port) {	/* If port number changes */
-			p->close_port(p);
-		}
-	}
-
 	/* Make sure the port is open */
 	if (!p->is_open) {
-		if (p->debug) fprintf(stderr,"icoms: HID port needs opening\n");
+		a1logd(p->log, 8, "hid_open_port: about to open HID port '%s'\n",p->name);
 
-		if (p->ppath != NULL) {
-			if (p->ppath->path != NULL)
-				free(p->ppath->path);
-			free(p->ppath);
-		}
-
-		if (p->paths == NULL)
-			p->get_paths(p);
-
-		if (port <= 0 || port > p->npaths)
-			error("icoms - hid_open_port: port number out of range!");
-
-		if (p->paths[port-1]->hev == NULL)
-			error("icoms - hid_open_port: Not an HID port!");
-
-		if ((p->ppath = calloc(sizeof(icompath), 1)) == NULL)
-			error("calloc() failed on com port path");
-		*p->ppath = *p->paths[port-1];				/* Structure copy */
-		if ((p->ppath->path = strdup(p->paths[port-1]->path)) == NULL)
-			error("strdup() failed on com port path");
-		p->port = port;
-
-		if (p->debug) fprintf(stderr,"icoms: About to open HID port '%s'\n",p->ppath->path);
-
-		p->vid = p->ppath->vid;
-		p->pid = p->ppath->pid;
-		p->hidd = p->ppath->hev;		/* A more convenient copy */
 		p->uflags = hidflags;
 
 #if defined(NT) 
@@ -536,18 +439,24 @@ char **pnames			/* List of process names to try and kill before opening */
 				      0, NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL))
 				                                              != INVALID_HANDLE_VALUE) {
 					memset(&p->hidd->ols,0,sizeof(OVERLAPPED));
-  					if ((p->hidd->ols.hEvent = CreateEvent(NULL, 0, 0, NULL)) == NULL)
-						error("Failed to create HID Event'\n");
+  					if ((p->hidd->ols.hEvent = CreateEvent(NULL, 0, 0, NULL)) == NULL) {
+						a1loge(p->log, ICOM_SYS, "hid_open_port: Failed to create HID "
+						                              "Event with %d'\n",GetLastError());
+						return ICOM_SYS;
+					}
 					break;
 				}
 				if (tries > 0 && pnames != NULL) {
 					/* Open failed. This could be the i1ProfileTray.exe */
-					kill_nprocess(pnames, p->debug);
+					kill_nprocess(pnames, p->log);
 					msec_sleep(100);
 				}
 			}
-			if (p->hidd->fh == INVALID_HANDLE_VALUE)
-				error("Failed to open HID device '%s'\n",p->ppath->path);
+			if (p->hidd->fh == INVALID_HANDLE_VALUE) {
+				a1loge(p->log, ICOM_SYS, "hid_open_port: Failed to open "
+				                     "HID '%s' with %d\n",GetLastError());
+				return ICOM_SYS;
+			}
 		}
 #endif /* NT */
 
@@ -558,42 +467,55 @@ char **pnames			/* List of process names to try and kill before opening */
 			SInt32 score;
 
 			if ((result = IOCreatePlugInInterfaceForService(p->hidd->ioob, kIOHIDDeviceUserClientTypeID,
-			    kIOCFPlugInInterfaceID, &piif, &score) != kIOReturnSuccess || piif == NULL))
-				error("Failed to open HID device '%s', result 0x%x, piif 0x%x\n",p->ppath->path,result,piif);
+			    kIOCFPlugInInterfaceID, &piif, &score) != kIOReturnSuccess || piif == NULL)) {
+				a1loge(p->log, ICOM_SYS, "hid_open_port: Failed to get piif for "
+				  "HID device '%s', result 0x%x, piif 0x%x\n",p->name,result,piif);
+				return ICOM_SYS;
+			}
 	
 			p->hidd->device = NULL;
 			if ((*piif)->QueryInterface(piif,
 				    CFUUIDGetUUIDBytes(kIOHIDDeviceInterfaceID122), (LPVOID)&p->hidd->device)
-			        != kIOReturnSuccess || p->hidd->device == NULL)
-				error("Getting HID device '%s' failed",p->ppath->path);
+			        != kIOReturnSuccess || p->hidd->device == NULL) {
+				a1loge(p->log, ICOM_SYS, "hid_open_port: Getting HID device '%s' failed",p->name);
+				return ICOM_SYS;
+			}
 			(*piif)->Release(piif);		/* delete intermediate object */
 
 			if ((*p->hidd->device)->open(p->hidd->device, kIOHIDOptionsTypeSeizeDevice)
-			                                                       != kIOReturnSuccess)
-				error("Opening HID device '%s' failed",p->ppath->path);
+			                                                       != kIOReturnSuccess) {
+				a1loge(p->log, ICOM_SYS, "hid_open_port: Opening HID device '%s' failed",p->name);
+				return ICOM_SYS;
+			}
 
 			/* Setup to handle interrupt read callbacks */
 			p->hidd->port = 0;
 			if ((*(p->hidd->device))->createAsyncPort(p->hidd->device, &p->hidd->port)
-			        != kIOReturnSuccess || p->hidd->port == 0)
-				error("Creating port on HID device '%s' failed",p->ppath->path);
+			        != kIOReturnSuccess || p->hidd->port == 0) {
+				a1loge(p->log, ICOM_SYS, "hid_open_port: Creating port on "
+				                          "HID device '%s' failed",p->name);
+				return ICOM_SYS;
+			}
 
 			p->hidd->evsrc = NULL;
 			if ((*(p->hidd->device))->createAsyncEventSource(p->hidd->device, &p->hidd->evsrc)
-			        != kIOReturnSuccess || p->hidd->evsrc == NULL)
-				error("Creating event source on HID device '%s' failed",p->ppath->path);
+			        != kIOReturnSuccess || p->hidd->evsrc == NULL) {
+				a1loge(p->log, ICOM_SYS, "hid_open_port: Creating event source on "
+				                                  "HID device '%s' failed",p->name);
+				return ICOM_SYS;
+			}
 
 		}
 #endif /* __APPLE__ */
 
-		p->is_usb = 0;
-		p->is_hid = 1;
 		p->is_open = 1;
-		if (p->debug) fprintf(stderr,"icoms: HID port is now open\n");
+		a1logd(p->log, 8, "hid_open_port: HID port is now open\n");
 	}
 
 	/* Install the cleanup signal handlers, and add to our cleanup list */
 	usb_install_signal_handlers(p);
+
+	return ICOM_OK;
 }
 
 /* ========================================================= */
@@ -614,7 +536,7 @@ UInt32 size
 ) {
 	icoms *p = (icoms *)target;
 
-//printf("\n~1 callback called with size %d, result 0x%x\n",size,result);
+	a1logd(p->log, 8, "HID callback called with size %d, result 0x%x\n",size,result);
 	p->hidd->result = result;
 	p->hidd->bread = size;
 	CFRunLoopStop(p->hidd->rlr);		/* We're done */
@@ -622,46 +544,47 @@ UInt32 size
 
 #endif /* __APPLE__ */
 
-/* HID Interrupt pipe Read  - thread friendly */
-/* Return error code (don't set error state). */
+/* HID Interrupt pipe Read */
 /* Don't retry on a short read, return ICOM_SHORT. */
 /* [Probably uses the control pipe. We need a different */
 /*  call for reading messages from the interrupt pipe] */
 static int
-icoms_hid_read_th(icoms *p,
+icoms_hid_read(icoms *p,
 	unsigned char *rbuf,	/* Read buffer */
 	int bsize,				/* Bytes to read */
 	int *breadp,			/* Bytes read */
-	double tout,			/* Timeout in seconds */
-	int debug,				/* debug flag value */
-	int *cut,				/* Character that caused termination */
-	int checkabort			/* Check for abort from keyboard */
+	double tout				/* Timeout in seconds */
 ) {
-	int lerr = 0;				/* Last error */
+	int retrv = ICOM_OK;	/* Returned error value */
 	int bread = 0;
 
-	if (!p->is_open)
-		error("icoms_hid_read: not initialised");
+	if (!p->is_open) {
+		a1loge(p->log, ICOM_SYS, "icoms_hid_read: device not initialised\n");
+		return ICOM_SYS;
+	}
 
 #if defined(NT)
 	{
 		unsigned char *rbuf2;
 
 		/* Create a copy of the data recieved with one more byte */
-		if ((rbuf2 = malloc(bsize + 1)) == NULL)
-			error("icoms_hid_read, malloc failed");
+		if ((rbuf2 = malloc(bsize + 1)) == NULL) {
+			a1loge(p->log, ICOM_SYS, "icoms_hid_read: malloc failed\n");
+			return ICOM_SYS;
+		}
 		rbuf2[0] = 0;
 		if (ReadFile(p->hidd->fh, rbuf2, bsize+1, (LPDWORD)&bread, &p->hidd->ols) == 0)  {
 			if (GetLastError() != ERROR_IO_PENDING) {
-				lerr = ICOM_USBR; 
+				retrv = ICOM_USBR; 
 			} else {
 				int res;
 				res = WaitForSingleObject(p->hidd->ols.hEvent, (int)(tout * 1000.0 + 0.5));
-				if (res == WAIT_FAILED)
-					error("HID wait on read failed");
-				else if (res == WAIT_TIMEOUT) {
+				if (res == WAIT_FAILED) {
+					a1loge(p->log, ICOM_SYS, "icoms_hid_read: HID wait on read failed\n");
+					return ICOM_SYS;
+				} else if (res == WAIT_TIMEOUT) {
 					CancelIo(p->hidd->fh);
-					lerr = ICOM_TO; 
+					retrv = ICOM_TO; 
 					bread = 0;
 				} else {
 					bread = p->hidd->ols.InternalHigh;
@@ -675,8 +598,8 @@ icoms_hid_read_th(icoms *p,
 	}
 #endif /* NT */
 
-#ifdef ENABLE_USB
 #ifdef __APPLE__
+#ifndef NEVER
 	{
 		IOReturn result;
 
@@ -685,112 +608,122 @@ icoms_hid_read_th(icoms *p,
 		p->hidd->bread = 0;
 
 		if ((*(p->hidd->device))->setInterruptReportHandlerCallback(p->hidd->device,
-			rbuf, bsize, hid_read_callback, (void *)p, NULL) != kIOReturnSuccess)
-			error("Setting callback handler for HID '%s' failed", p->ppath->path);
-		if ((*(p->hidd->device))->startAllQueues(p->hidd->device) != kIOReturnSuccess)
-			error("Starting queues for HID '%s' failed", p->ppath->path);
-
+			rbuf, bsize, hid_read_callback, (void *)p, NULL) != kIOReturnSuccess) {
+			a1loge(p->log, ICOM_SYS, "icoms_hid_read: Setting callback handler for "
+			                                             "HID '%s' failed\n", p->name);
+			return ICOM_SYS;
+		}
 		/* Call runloop, but exit after handling one callback */
 		p->hidd->rlr = CFRunLoopGetCurrent();
 		CFRunLoopAddSource(p->hidd->rlr, p->hidd->evsrc, kCFRunLoopDefaultMode);
+
+		if ((*(p->hidd->device))->startAllQueues(p->hidd->device) != kIOReturnSuccess) {
+			a1loge(p->log, ICOM_SYS, "icoms_hid_read: Starting queues for "
+			                                   "HID '%s' failed\n", p->name);
+			return ICOM_SYS;
+		}
+
 		result = CFRunLoopRunInMode(kCFRunLoopDefaultMode, tout, false);
-		if ((*(p->hidd->device))->stopAllQueues(p->hidd->device) != kIOReturnSuccess)
-			error("Stopping queues for HID '%s' failed", p->ppath->path);
+
+		if ((*(p->hidd->device))->stopAllQueues(p->hidd->device) != kIOReturnSuccess) {
+			a1loge(p->log, ICOM_SYS, "icoms_hid_read: Stopping queues for "
+			                                    "HID '%s' failed\n", p->name);
+			return ICOM_SYS;
+		}
 		if (result == kCFRunLoopRunTimedOut) {
-			lerr = ICOM_TO; 
+			retrv = ICOM_TO; 
 		} else if (result != kCFRunLoopRunStopped) {
-			lerr = ICOM_USBR; 
+			retrv = ICOM_USBR; 
 		}
 		CFRunLoopRemoveSource(p->hidd->rlr, p->hidd->evsrc, kCFRunLoopDefaultMode);
 
 		if (p->hidd->result == -1) {		/* Callback wasn't called */
-			lerr = ICOM_TO; 
+			retrv = ICOM_TO; 
 		} else if (p->hidd->result != kIOReturnSuccess) {
-			error("Callback for HID '%s' got unexpected return value", p->ppath->path);
+			a1loge(p->log, ICOM_SYS, "icoms_hid_read: Callback for "
+			        "HID '%s' got unexpected return value\n", p->name);
+			return ICOM_SYS;
 		}
 		bread = p->hidd->bread;
 	}
-#endif /* __APPLE__ */
-
-	if (checkabort) {		/* Check for user abort */
-		int c;
-		if ((c = poll_con_char()) != 0 && p->uih[c] != ICOM_OK) {
-//printf("~1 got for user abort with char 0x%x, code 0x%x\n",c,p->uih[c]);
-			*cut = c;
-			lerr |= p->uih[c];
+#else	// NEVER
+	/* This doesn't work. Don't know why */
+	/* Returns 0xe000404f = kIOUSBPipeStalled, Pipe has stalled, error needs to be cleared */
+	{
+		IOReturn result;
+		if ((result = (*p->hidd->device)->getReport(
+			p->hidd->device,
+		    kIOHIDReportTypeInput, 
+			9, 				/* Bulk or Interrupt transfer */
+			(void *)rbuf, 
+			(UInt32 *)&bsize, 
+			(unsigned int)(tout * 1000.0 + 0.5), 
+			NULL, NULL, NULL)
+		) != kIOReturnSuccess) {
+			/* (We could detect other error codes and translate them to ICOM) */
+			if (result == kIOReturnTimeout)
+				retrv = ICOM_TO; 
+			else
+				retrv = ICOM_USBW; 
+		} else {
+			bread = bsize;
 		}
 	}
+#endif	// NEVER
+#endif /* __APPLE__ */
 
 	if (breadp != NULL)
 		*breadp = bread;
 
-#else /* !ENABLE_USB */
-	lerr = ICOM_NOTS;
-#endif /* !ENABLE_USB */
+	a1logd(p->log, 8, "icoms_hid_read: About to return hid read %d bytes, ICOM err 0x%x\n",
+	                                                                         bread, retrv);
 
-	if (debug) fprintf(stderr,"icoms: About to return hid read %d bytes, ICOM err 0x%x\n",bread, lerr);
-
-	return lerr;
-}
-
-/* HID Read */
-/* Same as above, but set error state */
-static int
-icoms_hid_read(
-	icoms *p,
-	unsigned char *rbuf,	/* Read buffer */
-	int rsize,				/* Bytes to read */
-	int *bread,				/* Bytes read */
-	double tout				/* Timeout in seconds */
-) {
-	p->lerr = icoms_hid_read_th(p, rbuf, rsize, bread, tout, p->debug, &p->cut, 1);
-
-	return p->lerr;
+	return retrv;
 }
 
 /* - - - - - - - - - - - - - */
 
-/* HID Command Pipe Write  - thread friendly */
-/* Return error code (don't set error state). */
+/* HID Command Pipe Write */
 /* Don't retry on a short read, return ICOM_SHORT. */
 static int
-icoms_hid_write_th(icoms *p,
+icoms_hid_write(icoms *p,
 	unsigned char *wbuf,	/* Write buffer */
 	int bsize,				/* Bytes to write */
 	int *bwrittenp,			/* Bytes written */
-	double tout,			/* Timeout in seconds */
-	int debug,				/* debug flag value */
-	int *cut,				/* Character that caused termination */
-	int checkabort			/* Check for abort from keyboard */
+	double tout				/* Timeout in seconds */
 ) {
-	int lerr = 0;				/* Last error */
+	int retrv = ICOM_OK;	/* Returned error value */
 	int bwritten = 0;
 
-	if (!p->is_open)
-		error("icoms_hid_write: not initialised");
-
-#ifdef ENABLE_USB
+	if (!p->is_open) {
+		a1loge(p->log, ICOM_SYS, "icoms_hid_write: device not initialised\n");
+		return ICOM_SYS;
+	}
 
 #if defined(NT)
 	{
 		unsigned char *wbuf2;
 
 		/* Create a copy of the data to send with one more byte */
-		if ((wbuf2 = malloc(bsize + 1)) == NULL)
-			error("icoms_hid_write, malloc failed");
+		if ((wbuf2 = malloc(bsize + 1)) == NULL) {
+			a1loge(p->log, ICOM_SYS, "icoms_hid_write: malloc failed\n");
+			return ICOM_SYS;
+		}
 		memmove(wbuf2+1,wbuf,bsize);
 		wbuf2[0] = 0;		/* Extra report ID byte (why ?) */
 		if (WriteFile(p->hidd->fh, wbuf2, bsize+1, (LPDWORD)&bwritten, &p->hidd->ols) == 0) { 
 			if (GetLastError() != ERROR_IO_PENDING) {
-				lerr = ICOM_USBW; 
+				retrv = ICOM_USBW; 
 			} else {
 				int res;
 				res = WaitForSingleObject(p->hidd->ols.hEvent, (int)(tout * 1000.0 + 0.5));
-				if (res == WAIT_FAILED)
-					error("HID wait on write failed");
+				if (res == WAIT_FAILED) {
+					a1loge(p->log, ICOM_SYS, "icoms_hid_write: HID wait on write failed\n");
+					return ICOM_SYS;
+				}
 				else if (res == WAIT_TIMEOUT) {
 					CancelIo(p->hidd->fh);
-					lerr = ICOM_TO; 
+					retrv = ICOM_TO; 
 					bwritten = 0;
 				} else {
 					bwritten = p->hidd->ols.InternalHigh;
@@ -810,102 +743,124 @@ icoms_hid_write_th(icoms *p,
 		if ((result = (*p->hidd->device)->setReport(
 			p->hidd->device,
 		    kIOHIDReportTypeOutput, 
-			9, 
+			9, 				/* Bulk or Interrupt transfer */
 			(void *)wbuf, 
 			bsize, 
 			(unsigned int)(tout * 1000.0 + 0.5), 
 			NULL, NULL, NULL)) != kIOReturnSuccess) {
 			/* (We could detect other error codes and translate them to ICOM) */
 			if (result == kIOReturnTimeout)
-				lerr = ICOM_TO; 
+				retrv = ICOM_TO; 
 			else
-				lerr = ICOM_USBW; 
+				retrv = ICOM_USBW; 
 		} else {
 			bwritten = bsize;
 		}
 	}
 #endif /* __APPLE__ */
 
-	if (checkabort) {	/* Check for user abort */
-		int c;
-		if ((c = poll_con_char()) != 0 && p->uih[c] != ICOM_OK) {
-//printf("~1 got for user abort with char 0x%x, code 0x%x\n",c,p->uih[c]);
-			*cut = c;
-			lerr |= p->uih[c];
-		}
-	}
-
 	if (bwrittenp != NULL)
 		*bwrittenp = bwritten;
 
-#else /* !ENABLE_USB */
-	lerr = ICOM_NOTS;
-#endif /* !ENABLE_USB */
+	a1logd(p->log, 8, "icoms_hid_write: wrote %d bytes, ICOM err 0x%x\n",bwritten, retrv);
 
-	if (debug) fprintf(stderr,"icoms: About to return hid write %d bytes, ICOM err 0x%x\n",bwritten, lerr);
-
-	return lerr;
+	return retrv;
 }
 
-/* HID Write */
-/* Same as above, but set error state */
-static int
-icoms_hid_write(
-	icoms *p,
-	unsigned char *wbuf,	/* Read buffer */
-	int wsize,				/* Bytes to write */
-	int *bwritten,			/* Bytes written */
-	double tout				/* Timeout in seconds */
-) {
-	p->lerr = icoms_hid_write_th(p, wbuf, wsize, bwritten, tout, p->debug, &p->cut, 1);
-	return p->lerr;
-}
-
-
 /* ------------------------------------------------- */
 /* Set the hid port number and characteristics. */
 /* This may be called to re-establish a connection that has failed */
-static void
+/* return icom error */
+static int
 icoms_set_hid_port(
 icoms *p, 
-int    port,			/* HID com port, 1 - N, 0 for no change. */
 icomuflags hidflags,	/* Any special handling flags */
 int retries,			/* > 0 if we should retry set_configuration (100msec) */
 char **pnames			/* List of process names to try and kill before opening */
 ) {
-	if (p->debug) fprintf(stderr,"icoms: About to set hid port characteristics\n");
+	int rv = ICOM_OK;
+
+	a1logd(p->log, 8, "icoms_set_hid_port: About to set HID port characteristics\n");
 
 	if (p->is_open) 
 		p->close_port(p);
 
-	if (p->is_hid_portno(p, port) != instUnknown) {
+	if (p->port_type(p) == icomt_hid) {
 
-		hid_open_port(p, port, hidflags, retries, pnames);
+		if ((rv = hid_open_port(p, hidflags, retries, pnames)) != ICOM_OK)
+			return rv;
 
 		p->write = NULL;
 		p->read = NULL;
 
 	}
-	if (p->debug) fprintf(stderr,"icoms: hid port characteristics set ok\n");
+	a1logd(p->log, 8, "icoms_set_hid_port: HID port characteristics set ok\n");
 
-	// ~~~999
-//	usb_set_debug(8);
+	return rv;
 }
 
+/* Copy hid_idevice contents from icompaths to icom */
+/* return icom error */
+int hid_copy_hid_idevice(icoms *d, icompath *s) {
+
+	if (s->hidd == NULL) { 
+		d->hidd = NULL;
+		return ICOM_OK;
+	}
+
+	if ((d->hidd = calloc(sizeof(struct hid_idevice), 1)) == NULL) {
+		a1loge(d->log, ICOM_SYS, "hid_copy_hid_idevice: malloc failed\n");
+		return ICOM_SYS;
+	}
+
+#if defined(NT)
+	if ((d->hidd->dpath = strdup(s->hidd->dpath)) == NULL) {
+		a1loge(d->log, ICOM_SYS, "hid_copy_hid_idevice: malloc\n");
+		return ICOM_SYS;
+	}
+#endif
+#if defined(__APPLE__)
+	d->hidd->ioob = s->hidd->ioob;
+	IOObjectRetain(d->hidd->ioob);
+#endif
+#if defined (UNIX_X11)
+#endif
+	return ICOM_OK;
+}
+
+/* Cleanup and then free a hid_del_hid_idevice */
+void hid_del_hid_idevice(struct hid_idevice *hidd) {
+	if (hidd == NULL)
+		return;
+
+#if defined(NT)
+	if (hidd->dpath != NULL)
+		free(hidd->dpath);
+#endif
+#if defined(__APPLE__)
+	if (hidd->ioob != 0)
+		IOObjectRelease(hidd->ioob);
+#endif
+#if defined (UNIX_X11)
+#endif
+	free(hidd);
+}
+
+/* Cleanup any HID specific icoms info */
+void hid_del_hid(icoms *p) {
+	hid_del_hid_idevice(p->hidd);
+}
+
+
 /* ---------------------------------------------------------------------------------*/
 
 /* Set the HID specific icoms methods */
 void hid_set_hid_methods(
 icoms *p
 ) {
-	p->is_hid_portno  = hid_is_hid_portno;
 	p->set_hid_port   = icoms_set_hid_port;
-	p->hid_read_th    = icoms_hid_read_th;
-	p->hid_write_th   = icoms_hid_write_th;
 	p->hid_read       = icoms_hid_read;
 	p->hid_write      = icoms_hid_write;
-
-	p->reset_uih(p);
 }
 
 /* ---------------------------------------------------------------------------------*/
