@@ -8,7 +8,7 @@
  * Author: Graeme W. Gill
  * Date:   7/10/2007
  *
- * Copyright 2006 - 2007, Graeme W. Gill
+ * Copyright 2006 - 2013, Graeme W. Gill
  * All rights reserved.
  *
  * (Based on huey.h)
@@ -45,10 +45,6 @@
 #define I1D3_COMS_FAIL				0x62		/* Communication failure */
 #define I1D3_UNKNOWN_MODEL			0x63		/* Not an i1d3 */
 #define I1D3_DATA_PARSE_ERROR  		0x64		/* Read data parsing error */
-#define I1D3_USER_ABORT			    0x65		/* User hit abort */
-#define I1D3_USER_TERM		    	0x66		/* User hit terminate */
-#define I1D3_USER_TRIG 			    0x67		/* User hit trigger */
-#define I1D3_USER_CMND		    	0x68		/* User hit command */
 
 /* Real error code */
 #define I1D3_OK   					0x00
@@ -76,13 +72,15 @@
 #define I1D3_BAD_LED_MODE 			0x24
 #define I1D3_NO_COMS				0x25
 #define I1D3_BAD_STATUS				0x26
+#define I1D3_INT_THREADFAILED       0x27
 
 /* Sub-type of instrument */
 typedef enum {
 	i1d3_disppro    = 0,	/* i1 DisplayPro */
 	i1d3_munkdisp   = 1,	/* ColorMunki Display */
 	i1d3_oem        = 2,	/* OEM */
-	i1d3_nec_ssp    = 3 	/* NEC SpectraSensor Pro */
+	i1d3_nec_ssp    = 3, 	/* NEC SpectraSensor Pro */
+	i1d3_quato_sh3  = 4 	/* Quato Silver Haze 3 */
 } i1d3_dtype;
 
 /* Measurement mode */
@@ -96,16 +94,18 @@ typedef enum {
 struct _i1d3 {
 	INST_OBJ_BASE
 
-	inst_mode mode;				/* Currently selected mode */
+	amutex lock;				/* Command lock */
 
-	inst_opt_mode trig;			/* Reading trigger mode */
-	int trig_return;			/* Emit "\n" after trigger */
+	/* Modes */
+	inst_mode mode;				/* Currently selected mode */
+	inst_opt_type trig;			/* Reading trigger mode */
 
 	/* Information and EEPROM values */
 	i1d3_dtype dtype;			/* Base type of instrument, ie i1d3_disppro or i1d3_munkdisp */
 	i1d3_dtype stype;			/* Sub type of instrument, ie. any of i1d3_dtype. */
 								/* (Only accurate if it needed unlocking). */
 	int status;					/* 0 if status is ok (not sure what this is) */
+	char serial_no[21];			/* "I1-11.A-01.100999.02" or "CM-11.A-01.100999.02" */
 	char prod_name[32];			/* "i1Display3 " or "ColorMunki Display" */
 	int prod_type;				/* 16 bit product type number. i1d3_disppro = 0x0001, */
 								/* i1d3_munkdisp = 0x0002 */
@@ -121,24 +121,40 @@ struct _i1d3 {
 	double emis_cal[3][3];		/* Current emssion calibration matrix */
 	double ambi_cal[3][3];		/* Current ambient calibration matrix */
 
+	inst_disptypesel *dtlist;	/* Display Type list */
+	int ndtlist;				/* Number of valid dtlist entries */
+	int icx;					/* Internal calibration matrix index, 11 = Raw */
+	int cbid;					/* calibration base ID, 0 if not a base */
+	int refrmode;				/* nz if in refresh display mode/double int. time */
+	double ccmat[3][3];			/* Optional colorimeter correction matrix */
+	icxObserverType obType;		/* ccss observer to use */
+	xspect custObserver[3];		/* Custom ccss observer to use */
+
 	/* Computed factors and state */
-	int refmode;				/* nz if in refresh display mode double int. time */
-	int rrset;					/* Flag, nz if the refresh rate has been determined */
+	int    rrset;				/* Flag, nz if the refresh rate has been determined */
 	double refperiod;			/* if > 0.0 in refmode, target int time quantization */
+	double refrate;				/* Measured refresh rate in Hz */
+	int    refrvalid;			/* nz if refrate is valid */
 	double clk_freq;			/* Clock frequency (12Mhz) */
 	double dinttime;			/* default integration time = 0.2 seconds */
 	double inttime;				/* current integration time = 0.2 seconds */
 
-	double ccmat[3][3];			/* Optional colorimeter correction matrix */
+	double transblend;			/* Blend between fixed and adaptive integration */
+								/* at low light levels */
 
 	/* Other state */
 	int     led_state;			/* : Current LED on/off state */
 	double	led_period, led_on_time_prop, led_trans_time_prop;	/* Pulse state */
 
+	athread *th;                /* Diffuser position monitoring thread */
+	volatile int th_term;		/* nz to terminate thread */
+	volatile int th_termed;		/* nz when thread terminated */
+	int dpos;					/* Diffuser position, 0 = display, 1 = ambient */
+
 }; typedef struct _i1d3 i1d3;
 
 /* Constructor */
-extern i1d3 *new_i1d3(icoms *icom, instType itype, int debug, int verb);
+extern i1d3 *new_i1d3(icoms *icom, instType itype);
 
 
 #define I1D3_H
